<!--
SPDX-FileCopyrightText: 2023 Helge

SPDX-License-Identifier: MIT
-->

<!DOCTYPE html>
<html>

<head>
    <meta charset="utf-8" />
    <title>Verify your Fediverse Actor</title>
    <link rel="stylesheet" href="static/styles.css">
    <link rel="icon" href="static/icon.svg">
    <meta name="description"
        content="Tool to check if a Fediverse Actor satisfies the minimal requirements necessary to federate.">
    <meta property="og:title" content="Verify your Fediverse Actor" />
    <meta property="og:type" content="website" />
    <meta property="og:url" content="https://verify.funfedi.dev/" />
    <meta property="og:image" content="https://funfedi.dev/assets/FediverseLogo.png" />
</head>

<body>
    <main>
        <h1>Verify your Fediverse Actor</h1>

        <form action="/" method="GET">
            <label for="actor_uri">Actor URI to verify</label>
            <input type="text" name="actor_uri" id="actor_uri" />
            <details {% if has_restriction %} open {% endif %}>
                <summary>Options</summary>
                <ul>
                    {% for name in test_actors %}
                    <li><input type="checkbox" id="only_{{name}}" name="only_{{name}}" /><label for="only_{{name}}">Only
                            run for {{name}}</label>

                    </li>
                    {% endfor %}
                </ul>
            </details>
            <input type="submit" value="Verify" />
        </form>

        <p>
            Processing might take some time, please be patient.<br />
            Support actor URI formats are:
        <ul>
            <li>HTTP(s) URI, e.g.
                <code>http://pasture_verify_actor/alice</code> or <code>https://mastodon.social/actor</code>
            </li>
            <li>Account URI using webfinger, e.g.
                <code>acct:alice@pasture_verify_actor</code>
            </li>
            <li>Fediverse type handle, e.g.
                <code>alice@pasture_verify_actor</code> or
                <code>@alice@pasture_verify_actor</code>
            </li>
        </ul>
        For more information see the <a href="{{remote_urls.documentation}}">Documentation</a>.
        Results for some Fediverse applications are collected <a href="{{remote_urls.support_table}}">here</a>.
        Please report issues <a href="{{remote_urls.repository}}">here</a>.</p>

        <p>

            This tool was created as part of <a href="https://nlnet.nl/project/FediverseTestFramework/">Fediverse Test
                Framework</a>.<br />

            A project funded through the <a href="https://nlnet.nl/core">NGI0 Core</a> Fund,
            a fund established by <a href="https://nlnet.nl/">NLnet</a> with financial support from
            the European Commission's <a href="https://ngi.eu/">Next Generation Internet</a> programme,
            under the aegis of DG Communications Networks, Content and Technology
            under grant agreement No 101092990.

        </p>
    </main>
</body>

</html>